@@ -32,16 +32,12 @@ static const int SN_COORD = 250;	/* coordinate signal-to-noise ratio */
 static const int SN_WIDTH = 100;	/* width signal-to-noise ratio */
 static const int SN_ORIENT = 10;	/* orientation signal-to-noise ratio */
 
-static const int bits_per_long = 8 * sizeof(long);
-
-static inline int nlongs(int nbit)
-{
-	return (nbit + bits_per_long - 1) / bits_per_long;
-}
+#define LONG_BITS (sizeof(long) * 8)
+#define NLONGS(x) (((x) + LONG_BITS - 1) / LONG_BITS)
 
 static inline int getbit(const unsigned long *map, int key)
 {
-	return (map[key / bits_per_long] >> (key % bits_per_long)) & 0x01;
+	return (map[key / LONG_BITS] >> (key % LONG_BITS)) & 0x01;
 }
 
 static int getabs(struct input_absinfo *abs, int key, int fd)
@@ -61,10 +57,10 @@ static struct input_absinfo *get_info(struct mtdev *dev, int code)
 		return NULL;
 
 	ix = mtdev_abs2mt(code);
-	if (ix < 11)
+	if (ix < LEGACY_API_NUM_MT_AXES)
 		return &dev->abs[ix];
 	else
-		return &dev->state->ext_abs[ix - 11];
+		return &dev->state->ext_abs[ix - LEGACY_API_NUM_MT_AXES];
 }
 
 static void set_info(struct mtdev *dev, int code,
@@ -82,9 +78,31 @@ static void default_fuzz(struct mtdev *dev, int code, int sn)
 	abs->fuzz = (abs->maximum - abs->minimum) / sn;
 }
 
+static int mtdev_set_slots(struct mtdev *dev, int fd)
+{
+	struct { unsigned code; int values[DIM_FINGER]; } req;
+	struct mtdev_state *state = dev->state;
+	int rc, i, s, nslot;
+
+	nslot = mtdev_get_abs_maximum(dev, ABS_MT_SLOT) + 1;
+
+	for (i = 0; i < MT_ABS_SIZE; i++) {
+		req.code = mtdev_mt2abs(i);
+		if (!mtdev_has_mt_event(dev, req.code))
+			continue;
+		SYSCALL(rc = ioctl(fd, EVIOCGMTSLOTS(sizeof(req)), &req));
+		if (rc < 0)
+			return rc;
+		for (s = 0; s < DIM_FINGER && s < nslot; s++)
+			set_sval(&state->data[s], i, req.values[s]);
+	}
+
+	return 0;
+}
+
 int mtdev_configure(struct mtdev *dev, int fd)
 {
-	unsigned long absbits[nlongs(ABS_MAX)];
+	unsigned long absbits[NLONGS(ABS_MAX)];
 	int rc, i;
 
 	SYSCALL(rc = ioctl(fd, EVIOCGBIT(EV_ABS, sizeof(absbits)), absbits));
@@ -118,6 +136,9 @@ int mtdev_configure(struct mtdev *dev, int fd)
 	default_fuzz(dev, ABS_MT_WIDTH_MINOR, SN_WIDTH);
 	default_fuzz(dev, ABS_MT_ORIENTATION, SN_ORIENT);
 
+	if (dev->has_slot)
+		mtdev_set_slots(dev, fd);
+
 	return 0;
 }
 
@@ -131,10 +152,10 @@ int mtdev_has_mt_event(const struct mtdev *dev, int code)
 		return 0;
 
 	ix = mtdev_abs2mt(code);
-	if (ix < 11)
+	if (ix < LEGACY_API_NUM_MT_AXES)
 		return dev->has_abs[ix];
 	else
-		return dev->state->has_ext_abs[ix - 11];
+		return dev->state->has_ext_abs[ix - LEGACY_API_NUM_MT_AXES];
 }
 
 int mtdev_get_abs_minimum(const struct mtdev *dev, int code)
@@ -178,10 +199,10 @@ void mtdev_set_mt_event(struct mtdev *dev, int code, int value)
 		return;
 
 	ix = mtdev_abs2mt(code);
-	if (ix < 11)
+	if (ix < LEGACY_API_NUM_MT_AXES)
 		dev->has_abs[ix] = value;
 	else
-		dev->state->has_ext_abs[ix - 11] = value;
+		dev->state->has_ext_abs[ix - LEGACY_API_NUM_MT_AXES] = value;
 }
 
 void mtdev_set_abs_maximum(struct mtdev *dev, int code, int value)
