@@ -31,6 +31,11 @@
 #include "evbuf.h"
 #include "match.h"
 
+#ifndef input_event_sec
+#define input_event_sec time.tv_sec
+#define input_event_usec time.tv_usec
+#endif
+
 static inline int istouch(const struct mtdev_slot *data,
 			  const struct mtdev *dev)
 {
@@ -251,7 +256,8 @@ static void push_slot_changes(struct mtdev_state *state,
 			count++;
 	if (!count)
 		return;
-	ev.time = syn->time;
+	ev.input_event_sec = syn->input_event_sec;
+	ev.input_event_usec = syn->input_event_usec;
 	ev.type = EV_ABS;
 	ev.code = ABS_MT_SLOT;
 	ev.value = slot;
@@ -359,17 +365,20 @@ int mtdev_init(struct mtdev *dev)
 
 int mtdev_open(struct mtdev *dev, int fd)
 {
-	int ret;
+	int ret = -EINVAL;
+
+	if (!dev || fd < 0)
+		goto error;
 	ret = mtdev_init(dev);
 	if (ret)
 		goto error;
 	ret = mtdev_configure(dev, fd);
 	if (ret)
-		goto mtdev;
+		goto error;
 	return 0;
- mtdev:
-	mtdev_close(dev);
+
  error:
+	mtdev_close(dev);
 	return ret;
 }
 
@@ -411,8 +420,10 @@ void mtdev_close_delete(struct mtdev *dev)
 
 void mtdev_close(struct mtdev *dev)
 {
-	free(dev->state);
-	memset(dev, 0, sizeof(struct mtdev));
+	if (dev) {
+		free(dev->state);
+		memset(dev, 0, sizeof(struct mtdev));
+	}
 }
 
 void mtdev_delete(struct mtdev *dev)
